@@ -34,6 +34,7 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <fcntl.h>
+#include <locale.h>
 
 #include "gdm-logout-action.h"
 
@@ -85,27 +86,46 @@ static GtkWidget* create_background()
     GdkPixbuf *tmp, *shot;
     GdkScreen *screen;
 
+    guchar *pixels, *p;
+    int x, y, width, height, rowstride;
+    gboolean has_alpha;
+
     screen = gdk_screen_get_default();
 
-    tmp = gdk_pixbuf_get_from_drawable( NULL,
-                                        gdk_get_default_root_window(),
-                                        NULL,
-                                        0, 0, 0, 0,
-                                        gdk_screen_get_width(screen),
-                                        gdk_screen_get_height(screen) );
-
-    shot = gdk_pixbuf_composite_color_simple( tmp,
-                                              gdk_screen_get_width(screen),
-                                              gdk_screen_get_height(screen),
-                                              GDK_INTERP_NEAREST,
-                          128, gdk_screen_get_width(screen),
-                          0x000000, 0x000000);
-    g_object_unref( tmp );
+    shot = gdk_pixbuf_get_from_drawable( NULL,
+                                         gdk_get_default_root_window(),
+                                         NULL,
+                                         0, 0, 0, 0,
+                                         gdk_screen_get_width(screen),
+                                         gdk_screen_get_height(screen) );
 
+    /* make the background darker */
+    pixels = gdk_pixbuf_get_pixels(shot);
+    width = gdk_pixbuf_get_width(shot);
+    height = gdk_pixbuf_get_height(shot);
+    has_alpha = gdk_pixbuf_get_has_alpha(shot);
+    rowstride = gdk_pixbuf_get_rowstride(shot);
+    
+    for (y = 0; y < height; y++)
+    {
+        p = pixels;
+        for (x = 0; x < width; x++)
+        {
+            p[0] = p[0] / 2;
+            p[1] = p[1] / 2;
+            p[2] = p[2] / 2;
+            if( has_alpha )
+                p += 4;
+            else
+                p += 3;
+        }
+        pixels += rowstride;
+    }
+     
     back = gtk_window_new( GTK_WINDOW_TOPLEVEL );
     gtk_widget_set_app_paintable( back, TRUE );
     gtk_widget_set_double_buffered( back, FALSE );
-    g_signal_connect( back, "expose-event", G_CALLBACK(on_expose), shot);
+    g_signal_connect( back, "expose-event", G_CALLBACK(on_expose), shot );
     g_object_weak_ref( back, (GWeakNotify)g_object_unref,  shot );
 
     gtk_window_fullscreen( back );
@@ -179,9 +199,9 @@ static gboolean xfsm_shutdown_helper_hal_check ()
      * interface without shutting down/rebooting now.
      */
     message = dbus_message_new_method_call ("org.freedesktop.Hal",
-                                                                                    "/org/freedesktop/Hal/devices/computer",
-                                                                                    "org.freedesktop.Hal.Device.SystemPowerManagement",
-                                                                                    "ThisMethodMustNotExistInHal");
+                                            "/org/freedesktop/Hal/devices/computer",
+                                            "org.freedesktop.Hal.Device.SystemPowerManagement",
+                                            "ThisMethodMustNotExistInHal");
     result = dbus_connection_send_with_reply_and_block (connection, message, 2000, &error);
     dbus_message_unref (message);
 
@@ -320,8 +340,10 @@ int main( int argc, char** argv )
     GPid pid;
     GOptionContext *context;
     GError* err = NULL;
+    gboolean composited;
 
 #ifdef ENABLE_NLS
+    setlocale(LC_ALL, "");
     bindtextdomain ( GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR );
     bind_textdomain_codeset ( GETTEXT_PACKAGE, "UTF-8" );
     textdomain ( GETTEXT_PACKAGE );
@@ -349,8 +371,11 @@ int main( int argc, char** argv )
 
     back = create_background();
 
+    /* check if the window is composited */
+    composited = gtk_widget_is_composited( back );
+
     gtk_icon_theme_append_search_path( gtk_icon_theme_get_default(),
-                                            PACKAGE_DATA_DIR "/images" );
+                                            PACKAGE_DATA_DIR "/lxsession/images" );
 
     dlg = gtk_dialog_new_with_buttons( _("Logout"), (GtkWindow*)back, GTK_DIALOG_MODAL,
                                                GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL );
@@ -433,9 +458,10 @@ int main( int argc, char** argv )
         gtk_box_pack_start( vbox, btn, FALSE, FALSE, 4 );
     }
 
-    /* If GDM is running */
+    /* If GDM or KDM is running */
     if( g_file_test("/var/run/gdm_socket", G_FILE_TEST_EXISTS)
-        || g_file_test("/tmp/.gdm_socket", G_FILE_TEST_EXISTS) )
+        || g_file_test("/tmp/.gdm_socket", G_FILE_TEST_EXISTS)
+        || g_file_test("/var/run/kdm.pid", G_FILE_TEST_EXISTS) )
     {
         btn = create_dlg_btn(_("S_witch User"), "gnome-session-switch", LOGOUT_ACTION_SWITCH_USER );
         gtk_box_pack_start( vbox, btn, FALSE, FALSE, 4 );
@@ -452,6 +478,7 @@ int main( int argc, char** argv )
 
     gdk_pointer_grab( dlg->window, TRUE, 0, NULL, NULL, GDK_CURRENT_TIME );
     gdk_keyboard_grab( dlg->window, TRUE, GDK_CURRENT_TIME );
+    if( !composited ) gdk_x11_grab_server();
 
     switch( (res = gtk_dialog_run( (GtkDialog*)dlg )) )
     {
@@ -469,7 +496,7 @@ int main( int argc, char** argv )
             gdk_keyboard_ungrab( GDK_CURRENT_TIME );
             return 0;
     }
-
+    if( !composited ) gdk_x11_ungrab_server();
     gdk_pointer_ungrab( GDK_CURRENT_TIME );
     gdk_keyboard_ungrab( GDK_CURRENT_TIME );
 
@@ -480,7 +507,10 @@ int main( int argc, char** argv )
     {
         if( res == LOGOUT_ACTION_SWITCH_USER )
         {
-            g_spawn_command_line_sync ("gdmflexiserver --startnew", NULL, NULL, NULL, NULL);
+	   	    if( g_file_test("/var/run/gdm_socket", G_FILE_TEST_EXISTS) || g_file_test("/tmp/.gdm_socket", G_FILE_TEST_EXISTS) )
+            	g_spawn_command_line_sync ("gdmflexiserver --startnew", NULL, NULL, NULL, NULL);
+            else if ( g_file_test("/var/run/kdm.pid", G_FILE_TEST_EXISTS) )
+            	g_spawn_command_line_sync ("kdmctl reserve", NULL, NULL, NULL, NULL);
             return 0;
         }
 
